use std::str::FromStr;

use i18n::{message, message_ids::MessageId};
use validator::{Validate, ValidationErrors};

use crate::model::response::{ResData, ResErr, ResResult};

/// 验证请求参数
pub fn validate_params(params: &impl Validate, local: &str) -> ResResult<ResData<()>> {
    let validate_err = params.validate();
    match validate_err {
        Ok(_) => Result::Ok(ResData::none()),
        Err(err) => {
            let mut errors = vec![];
            for (_, err) in err.errors() {
                match err {
                    validator::ValidationErrorsKind::Struct(err) => {
                        pack_error_msg(&mut errors, err, local);
                    },
                    validator::ValidationErrorsKind::List(err) => {
                        for (_, err) in err.iter() {
                            pack_error_msg(&mut errors, err, local);
                        }
                    },
                    validator::ValidationErrorsKind::Field(err) => {
                        err.iter().for_each(|e| {
                            let msg = e.message.clone().unwrap_or_default();
                            errors.push(message!(local, MessageId::from_str(msg.trim()).unwrap()))
                        });
                    },
                };
            }
            Err(ResErr::params(errors.join(";")))
        },
    }
}

fn pack_error_msg(errors: & mut Vec<&str>, raw_errors: &Box<ValidationErrors>, local: &str) {
    for (_field, errs) in raw_errors.field_errors() {
        for e in errs {
            let msg = e.message.clone().unwrap_or_default();
            errors.push(message!(local, MessageId::from_str(msg.trim()).unwrap()));
        }
    }
}